#boltzmann
# Numerical system which computes the discrete probability distribution 
# of energies shared amongst M molecules. Total energy is N units of a
# fixed 'quantum' of energy. Each molecule can have between 0 and N units
# of energy. The sum of energy within all M molecules is N.
#
# LAST UPDATED by Andy French November 2017

# https://docs.scipy.org/doc/numpy-dev/user/numpy-for-matlab-users.html

## IMPORT PYTHON LIBRARIES ##

import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as Npy

#

## USER DEFINED FUNCTIONS ##

#Line of best fit function yfit = m*x +c, with product moment correlation
#coefficient r
def bestfit(x,y):
    #Find any x or y values that are NaN or Inf
    test = Npy.logical_or( Npy.isnan(Npy.abs(x)),Npy.isnan(Npy.abs(y)) )
    test = Npy.logical_or( test, Npy.isinf(Npy.abs(x)) )
    test = Npy.logical_or( test, Npy.isinf(Npy.abs(y)) )
    i = Npy.nonzero(test)
    x = Npy.delete(x,i,None)
    y = Npy.delete(y,i,None)
    n = Npy.size(x)
    xbar = Npy.mean(x)
    ybar = Npy.mean(y)
    xybar = Npy.mean(x*y)
    Vx = Npy.mean(x**2 ) - xbar**2
    Vy = Npy.mean(y**2 ) - ybar**2
    COVxy = Npy.mean(x*y) - xbar*ybar
    m = COVxy/Vx
    c = ybar - m*xbar
    r = COVxy/Npy.sqrt( Vx*Vy )
    yfit = m*x + c
    xfit = x

    #Create figure and axis
    fig = plt.figure(figsize=(10, 8))

    #Plot P vs number of quanta of energy
    fsize = 18;
    plt.plot(x,y,xfit,yfit)
    plt.xlabel("x", size=fsize )
    plt.ylabel("y", size=fsize )
    plt.xticks(fontsize=fsize)
    plt.yticks(fontsize=fsize) 
    plot_title = "y = "+str(m)+"x + "+str(c)
    plt.title(plot_title, size=fsize )
    plt.grid(True)
    fig.savefig('best_fit.png', dpi=300)

    #Set output array
    output = [yfit,xfit,r,ybar,xbar,m,c]
    return output

#

#Modified logarithm function to avoid divide by zero error.
def log(x):
    #Initialize output
    y = Npy.linspace(0,1,Npy.size(x))
    
    #Negative x
    negatives = Npy.nonzero( x<0 )

    #Zero x
    zeros = Npy.nonzero( x==0 )

    #Positive x values
    positives = Npy.nonzero( x>0 )

    #Modify output
    y[negatives] = float("nan")
    y[zeros] = float("-Inf")
    y[positives] = Npy.log( x[positives] )
    return y
    
#

## INPUTS ##

# Number of energy quanta
N = 10000

# Number of molecules
M = 500

# Number of random 'deals' of quanta of energy amongst molecules
I = 100

# Fontsize for graphs
fsize = 18

# Max number of quanta for graph axis (about 3 mean quanta per molecule
# i.e. 5 * N /M is recommended).
nmax = int( Npy.ceil( 5*N/M ) )

#

#Display welcome message to console
print("Boltzmann distribution simulation")
print("By Dr Andrew French. November 2017 \n")
print("Energy quanta N = " + str(N))
print("Number of molecules M = " + str(M))
print("Number of iterations I = " + str(I))
print("\n Computing iterations ...")

#Intialize array of quanta of energy for each molecule.
E = Npy.zeros((I,M))

#Define a binary vector of N zeros and M-1 ones. Randomly shuffling this
#will define the allocation of N quanta of energy into M 'boxes'. A zero
#corresponds to a energy quantum, a 1 to a change to a new molecule
B1 = Npy.zeros((1,N))
B1 = B1[0]
B2 = Npy.ones((1,M-1))
B2 = B2[0]
B = Npy.hstack((B1,B2))

#Step through iterations and randomly deal out N quanta of energy
for i in range(0,I-1):

    #Shuffle B
    Npy.random.shuffle(B)
    
    #Step through b and work out distribution of energy quanta. Initialize
    #molecule number m = 0
    m = 0
    for j in range(0,N+M-2):
        if B[j]==0:
            #Add an energy quantum to molecule m
            E[i,m] = E[i,m] + 1
        else:
            #New molecule number
            m = m + 1

#All molecules are equivalent, so pool all iterations results into a single
#vector of energy quanta numbers per molecule
E = Npy.reshape(E,(1,Npy.size(E)))
E = E[0]

#Determine frequency table for energies
F = Npy.zeros((1,N+1))
F = F[0]
n = Npy.arange(0,N+1)
for i in range(0,N):
    F[i] = Npy.size( Npy.nonzero( E == n[i] ) )

#Scale frequency table such that total frequency = 1 i.e. create an array
#of probabilities of a particular number of quanta
P = F/Npy.sum(F)

#Truncate n and P
n = n[0:nmax+1]
P = P[0:nmax+1]

#Find a line of best fit for lnP vs n
b = bestfit(n,log(P))
yfit=b[0]
xfit=b[1]
r=b[2]
ybar=b[3]
xbar=b[4]
m=b[5]
c=b[6]

#Determine discrete Boltzmann distribution from line of best fit
nn = Npy.linspace(0,nmax+1,1000)
Z = Npy.exp(-c)
nbar = -1/m
Pboltz = Npy.exp( -nn/nbar )/Z

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot P vs number of quanta of energy
plt.plot(n,P,label = "Simulation")
plt.plot(nn,Pboltz,label = "Exp fit")
plt.xlabel("Number of energy quanta n", size=fsize )
plt.ylabel("Probability", size=fsize )
plot_title = "Boltzmann sim: iter=" + str(I) + ", N=" + str(N) + ", M=" + str(M) + ", nbar=" + str(Npy.around(nbar,2))
plt.title(plot_title, size=fsize )
plt.xticks(fontsize=fsize)
plt.yticks(fontsize=fsize) 
plt.xlim((0,nmax))
plt.grid(True)
plt.legend( fontsize = fsize,loc = "upper right" )
fig.savefig('boltzmann Python.png',dpi=300)

#Wait for user input to close terminal window
input("\n Press return to close ")

#End of code




