#kepler
# Model of a binary mass system undergoing mutually elliptical orbits
# LAST UPDATED by Dr Andrew French. November 2017

#Import Python numeric and plotting libraries
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as Npy

#

## FIXED INPUTS ##
    
#Define pi
pi = float(Npy.pi);

#Universal Gravitational constant / m**3kg**-1s**-2
G = 6.67384e-11;

#Mass of the Sun /kg
M_sol = 1.98855e30

#Mass of the Earth /kg
m_earth = 5.9722e24

#Earth Year /s
Yr = 365*24*3600

#Astronomical unit (Earth to Sun mean distance /m)
AU = 149597870700

#Number of data points
N = 100;

#Fontsize for graphs
fsize = 18;

#Font RGB colours for graps
col_M = [1,0,0]
col_m = [0,0,1]

#

#Display welcome message
print("\n Two body Kepler problem Python simulation")
print(" By Dr Andrew French. November 2017 \n")

#Get mass of star (M) in solar masses and convert to kg
M = input( " Define mass of star in solar masses: " )
if M=="":
    M = 1
    print(" M = 1 solar masses")
else:
    M = float(M)
M = M*M_sol

#Get mass of planet (m) in Earth masses and convert to kg
m = input( " Define mass of planet in earth masses: " )
if m=="":
    m = 1e5
    print(" m = 100,000 solar masses")
else:
    m = float(m)
m = m*m_earth

#Get semi-major axis of orbit in AU and convert to metres
a = input( " Define semi-major axis of orbit in AU: " )
if a=="":
    a = 3
    print(" a = 3 AU")
else:
    a = float(a)
a = a*AU

#Get eccentricity of (mutually) elliptic orbits
ecc = input( " Define eccentricity of elliptical orbits: " )
if ecc=="":
    ecc = 0.5
    print(" eccentricity = 0.5")
else:
    ecc = float(ecc)
if ecc > 0.9:
    ecc = 0.9
    print(" Maximum orbital eccentricity = 0.9")
elif ecc < 0:
    ecc = 0
    print(" Minimum orbital eccentricity = 0")

#Calculate orbital period /s and then convert to years
P = Npy.sqrt( ( (4*pi**2)/(G*(m+M)) )*(a**3) )

#Calculate orbital period in Earth days
Pdays = Npy.around( P/(24*3600) , 1)
print("\n Period = " + str(Pdays) + " Earth days")

#Vector of polar angles in radians
t = Npy.linspace(0,2*pi,N)
    
#Range /m
r = 1-ecc*Npy.cos(t)
r = a*(1-ecc**2)/r

#x and y coordinates /m
xs = (m/(M+m))*r*Npy.cos(t)
xp = -(M/(M+m))*r*Npy.cos(t)
ys = (m/(M+m))*r*Npy.sin(t)
yp = -(M/(M+m))*r*Npy.sin(t)

#Velocity /ms**-1
Vx = (1-ecc*Npy.cos(t))*Npy.sqrt( G*(m+M)/((1-ecc**2)*a) )*( -Npy.sin(t) - ( ecc*Npy.sin(t)/(1 - ecc*Npy.cos(t) ))*Npy.cos(t))
Vy = (1-ecc*Npy.cos(t))*Npy.sqrt( G*(m+M)/((1-ecc**2)*a) )*( Npy.cos(t) - ( ecc*Npy.sin(t)/(1 - ecc*Npy.cos(t) ))*Npy.sin(t)) 
vxs = (m/(M+m))*Vx
vys = (m/(M+m))*Vy
vxp = -(M/(M+m))*Vx
vyp = -(M/(M+m))*Vy
vs = Npy.sqrt( vxs**2 + vys**2 )
vp = Npy.sqrt( vxp**2 + vyp**2 )

#Determine maximum orbital speed for plot
vmax = Npy.around( Npy.nanmax( Npy.hstack(( vs/1e3,vp/1e3,[30] )) ), 0 )

#Plot x,y elliptical orbits and save as a 300dpi PNG file
figxy = plt.figure(figsize=(10, 8))
plt.plot(xs/AU,ys/AU,'.-', color = col_M, label="Star, mass M")
plt.plot(xp/AU,yp/AU,'.-', color = col_m, label="Planet, mass m")
plt.legend( fontsize = fsize,loc = "upper right", numpoints=1 )
plt.plot(xs[0]/AU,ys[0]/AU,'*', color = [0,1,0] )
plt.plot(xp[0]/AU,yp[0]/AU,'*', color = [1,1,0])
plt.xlabel("x/AU", size=fsize )
plt.ylabel("y/AU", size=fsize )
plt.xticks(fontsize = fsize )
plt.yticks(fontsize = fsize )
plt.title("M = " + str(M/M_sol) + " solar masses, m = " + str(m/m_earth) + " Earth masses \n P = " + str(Pdays) + " days", size=fsize )
plt.grid(True)
plt.axis('equal')
figxy.savefig('kepler xy.png', dpi=300)

#Plot orbital speeds /km per second vs angle /degrees
figv = plt.figure(figsize=(10, 8))
plt.plot(t*180/pi,vs/1e3,'-', color = col_M, label="Star, mass M")
plt.plot(t*180/pi,vp/1e3,'-', color = col_m, label="Planet, mass m")
plt.ylabel("Orbital speed / $\mathregular{kms^{-1}}$", size=fsize )
plt.xlabel("Polar angle /degrees", size=fsize )
plt.xticks(fontsize = fsize )
plt.yticks(fontsize = fsize )
plt.xlim( 0, 360 )
plt.ylim( 0, vmax )
plt.legend( fontsize = fsize,loc = "upper right" )
plt.title("M = " + str(M/M_sol) + " solar masses, m = " + str(m/m_earth) + " Earth masses \n P = " + str(Pdays) + " days", size=fsize )
plt.grid(True)
figv.savefig('kepler speeds.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press return to close ")

#End of code

