#solar_system
# Demonstration of the numeric solution of a simple orbits problem. In this case the motion
# of a set of planets about a star whose mass is significantly higher than the
# mass of any of the planets. We shall assume initial conditions that give rise
# to elliptical orbits about the coordinate origin, which is where the star
# is assumed to remain. Ignore any inter-planetary gravitational interactions.
#
# A Euler and Verlet methods will be contrasted with the exact solutions
#
# LAST UPDATED by Andy French December 2017
#
# NOTE ############# means hard coded lines that need to be modified

#Import Python numeric and plotting libraries
import numpy as Npy
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

#

## INPUTS ##

#Number of periods for each planet
num_P = 5

#Timestep for numeric calculation /Yrs
dt = 0.05

#Set planet configuration "All", "Earth & Mars", "Inner","Outer"
planet_option = "All"

#Include Euler method?
include_euler = "no"

#
    
#Define pi
pi = float(Npy.pi);

#Universal Gravitational constant / m**3kg**-1s**-2
G = 6.67384e-11;

#Mass of the Sun /kg
M = 1.98855e30

#Earth Year /s
Yr = 365*24*3600

#Astronomical unit (Earth to Sun mean distance /m)
AU = 149597870700

#Fontsize for graphs
fsize = 18;

#Names of the planets
if planet_option == "All":
    planets = ["Mercury","Venus","Earth","Mars","Jupiter","Saturn","Uranus","Neptune"]
elif planet_option == "Earth & Mars":
    planets = ["Earth","Mars"]
elif planet_option == "Inner":
    planets = ["Mercury","Venus","Earth","Mars"]
elif planet_option == "Outer":
    planets = ["Jupiter","Saturn","Uranus","Neptune"]
else:
    planets = ["Mercury","Venus","Earth","Mars","Jupiter","Saturn","Uranus","Neptune"]
        

#Semi-major axes of planets /AU
if planet_option == "All":
    a_planets = [0.387,0.723,1.00,1.523,5.202,9.576,19.293,30.246]
elif planet_option == "Earth & Mars":
    a_planets = [1.00,1.523]
elif planet_option == "Inner":
    a_planets = [0.387,0.723,1.00,1.523]
elif planet_option == "Outer":
    a_planets = [5.202,9.576,19.293,30.246]
else:
    a_planets = [0.387,0.723,1.00,1.523,5.202,9.576,19.293,30.246]

#Orbital eccentricities of planets
if planet_option == "All":
    ecc_planets = [ 0.21,0.01,0.02,0.09,0.05,0.06,0.05,0.01 ]
elif planet_option == "Earth & Mars":
    ecc_planets = [ 0.02,0.09 ]
elif planet_option == "Inner":
    ecc_planets = [ 0.21,0.01,0.02,0.09 ]
elif planet_option == "Outer":
    ecc_planets = [ 0.05,0.06,0.05,0.01 ]
else:
    ecc_planets = [ 0.21,0.01,0.02,0.09,0.05,0.06,0.05,0.01 ]

#Font RGB colours for planets
if planet_option == "All":
    col = [ [0.1,0.1,0.1],[0.5,0.5,0.5],[0,0,0.8],[1,0,0],[1,0,1],[1,1,0],[0,1,0],[0,0,1] ]
elif planet_option == "Earth & Mars":
    col = [ [0,0,0.8],[1,0,0] ]
elif planet_option == "Inner":
    col = [ [0.1,0.1,0.1],[0.5,0.5,0.5],[0,0,0.8],[1,0,0] ]
elif planet_option == "Outer":
    col = [ [1,0,1],[1,1,0],[0,1,0],[0,0,1] ]
else:
    col = [ [0.1,0.1,0.1],[0.5,0.5,0.5],[0,0,0.8],[1,0,0],[1,0,1],[1,1,0],[0,1,0],[0,0,1] ]
    
#Axis limits /AU
if planet_option == "All":
    xmax = 35
    ymax = 35
elif planet_option == "Earth & Mars":
    xmax = 2
    ymax = 2
elif planet_option == "Inner":
    xmax = 2
    ymax = 2
elif planet_option == "Outer":
    xmax = 32
    ymax = 32
else:
    xmax = 35
    ymax = 35

#

####################################

#Define gravity function
def g(t,x,y,vx,vy):
    r = Npy.sqrt( x**2 + y**2 )
    A = -G*M/( r**2 )
    costheta = x/r
    sintheta = y/r
    ax = A*costheta
    ay = A*sintheta
    return ax, ay
    
####################################
    
#

#Define elliptical orbit x,y and vx, vy parameters, and also period P
#Assume SI unit inputs
def elliptical( theta, a, ecc ):
    r = a*(1-ecc**2)/( 1-ecc*Npy.cos(theta) )
    x = r*Npy.cos(theta)
    y = r*Npy.sin(theta)
    v = (1-ecc*Npy.cos(theta))*Npy.sqrt( G*M/( (1-ecc**2)*a ) )
    thetahatx = -Npy.sin(theta)
    thetahaty = Npy.cos(theta)
    rhatx = Npy.cos(theta)
    rhaty = Npy.sin(theta)
    vx = v*( thetahatx - rhatx*ecc*Npy.sin(theta)/( 1-ecc*Npy.cos(theta) ) )
    vy = v*( thetahaty - rhaty*ecc*Npy.sin(theta)/( 1-ecc*Npy.cos(theta) ) )
    P = ( 2*pi/Npy.sqrt(G*M) )*(a**(1.5))
    return r, x, y, vx, vy, P
    
#

#Define Euler method solver
def euler( dt, t0, x0, y0, vx0, vy0, N ):
    #Initialize output vectors
    t = Npy.zeros((1,N))
    t = t[0]
    t[0] = t0
    x = Npy.zeros((1,N))
    x = x[0]
    x[0] = x0
    y = Npy.zeros((1,N))
    y = y[0]
    y[0] = y0
    vx = Npy.zeros((1,N))
    vx = vx[0]
    vx[0] = vx0
    vy = Npy.zeros((1,N))
    vy = vy[0]
    vy[0] = vy0
	
    #Loop through vectors to compute t, x, y, vx, vy
    for n in range(1,N):
        ax, ay = g( t[n-1],x[n-1],y[n-1],vx[n-1],vy[n-1] )
        t[n] = t[n-1] + dt
        vx[n] = vx[n-1] + ax*dt
        x[n] = x[n-1] + vx[n-1]*dt
        vy[n] = vy[n-1] + ay*dt
        y[n] = y[n-1] + vy[n-1]*dt
    
    #Set return values of function
    return t, x, y, vx, vy
    
#

#Define Verlet solver
def verlet( dt, t0, x0, y0, vx0, vy0, N ):
    #Initialize output vectors
    t = Npy.zeros((1,N))
    t = t[0]
    t[0] = t0
    x = Npy.zeros((1,N))
    x = x[0]
    x[0] = x0
    y = Npy.zeros((1,N))
    y = y[0]
    y[0] = y0
    vx = Npy.zeros((1,N))
    vx = vx[0]
    vx[0] = vx0
    vy = Npy.zeros((1,N))
    vy = vy[0]
    vy[0] = vy0
	
    #Loop through vectors to compute t, x, y, vx, vy
    for n in range(1,N):
        ax, ay = g( t[n-1],x[n-1],y[n-1],vx[n-1],vy[n-1] )
        t[n] = t[n-1] + dt
        x[n] = x[n-1] + vx[n-1]*dt + 0.5*ax*(dt**2)
        y[n] = y[n-1] + vy[n-1]*dt + 0.5*ay*(dt**2)
        axx, ayy = g( t[n],x[n],y[n],vx[n-1],vy[n-1] )
        vx[n] = vx[n-1] + 0.5*( ax + axx )*dt
        vy[n] = vy[n-1] + 0.5*( ay + ayy )*dt
    
    #Set return values of function
    return t, x, y, vx, vy

#

#Run simulation

#Message to command window
print("\n Solar System Simulation")
print(" By Dr Andy French. December 2017 ")
print(" Planet option = " + planet_option)
print(" Timestep dt = " + str(dt) + " years")

#Create figure and axis
fig1 = plt.figure(figsize=(10, 8))

#Plot x vs y for each planet
for i in range(0,len(planets)):
    
    #Set initial conditions
    r0, x0, y0, vx0, vy0, P = elliptical( 0, a_planets[i]*AU, ecc_planets[i] )
    t0 = 0
    
    #Compute exact orbital trajectory
    theta = Npy.linspace( 0, 2*pi*num_P, 1000 )
    rexact, xexact, yexact, vxexact, vyexact, P = elliptical( theta, a_planets[i]*AU, ecc_planets[i] )
    
    #Determine number of trajectory points
    N = int( num_P*P/(dt*Yr) )
    
    #Compute orbital trajectory using Euler method
    te, xe, ye, vxe, vye = euler( dt*Yr, t0, x0, y0, vx0, vy0, N )
    
    #Compute orbital trajectory using Verlet method
    t, x, y, vx, vy = verlet( dt*Yr, t0, x0, y0, vx0, vy0, N )
    
    #Plot trajectories
    if include_euler == "yes":
        plt.plot( xe/AU,ye/AU,'--',label = planets[i] + " Euler" , color = col[i] )
    plt.plot( x/AU,y/AU,'-.',label = planets[i] + " Verlet", color = col[i] )
    plt.plot( xexact/AU,yexact/AU,'-',label = planets[i] + " Exact", color = col[i] )

#General axis properties
plt.xlabel( "x /AU", size=fsize );
plt.ylabel( "y /AU", size=fsize );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( "Solar System Simulation. dt="+str(dt)+" years, "+str(num_P)+ " orbits", size=fsize )
plt.legend( fontsize = fsize/2,loc = "upper right", numpoints=1 )
plt.plot(0,0,'k*')
plt.axis('equal')
plt.xlim( -xmax, xmax )
plt.ylim( -ymax, ymax )
plt.grid(True)

#Export PNG file
fig1.savefig('Solar System ' + planet_option + '.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press return key to continue ...")

#End of code
